@@ -41,6 +41,9 @@ G_BEGIN_DECLS
 
 #define AGS_UI_PROVIDER_DEFAULT_TIMEOUT (1.0 / 25.0)
 #define AGS_UI_PROVIDER_DEFAULT_PADDING (6)
+#define AGS_UI_PROVIDER_DEFAULT_SPACING (6)
+#define AGS_UI_PROVIDER_DEFAULT_COLUMN_SPACING (6)
+#define AGS_UI_PROVIDER_DEFAULT_ROW_SPACING (6)
 #define AGS_UI_PROVIDER_DEFAULT_MARGIN_END (6)
 
 typedef struct _AgsUiProvider AgsUiProvider;
@@ -83,18 +86,6 @@ struct _AgsUiProviderInterface
   GtkWidget* (*get_window)(AgsUiProvider *ui_provider);
   void (*set_window)(AgsUiProvider *ui_provider,
 		     GtkWidget *window);
-
-  GtkWidget* (*get_automation_window)(AgsUiProvider *ui_provider);
-  void (*set_automation_window)(AgsUiProvider *ui_provider,
-				GtkWidget *automation_window);
-
-  GtkWidget* (*get_wave_window)(AgsUiProvider *ui_provider);
-  void (*set_wave_window)(AgsUiProvider *ui_provider,
-			  GtkWidget *wave_window);
-  
-  GtkWidget* (*get_sheet_window)(AgsUiProvider *ui_provider);
-  void (*set_sheet_window)(AgsUiProvider *ui_provider,
-			   GtkWidget *sheet_window);
   
   GtkWidget* (*get_export_window)(AgsUiProvider *ui_provider);
   void (*set_export_window)(AgsUiProvider *ui_provider,
@@ -132,8 +123,6 @@ struct _AgsUiProviderInterface
   void (*set_machine)(AgsUiProvider *ui_provider,
 		      GList *machine);
 
-  gboolean (*use_composite_editor)(AgsUiProvider *ui_provider);
-  
   GtkWidget* (*get_composite_editor)(AgsUiProvider *ui_provider);
   void (*set_composite_editor)(AgsUiProvider *ui_provider,
 			       GtkWidget *composite_editor);  
@@ -186,18 +175,6 @@ void ags_ui_provider_set_animation_window(AgsUiProvider *ui_provider,
 GtkWidget* ags_ui_provider_get_window(AgsUiProvider *ui_provider);
 void ags_ui_provider_set_window(AgsUiProvider *ui_provider,
 				GtkWidget *window);
-
-GtkWidget* ags_ui_provider_get_automation_window(AgsUiProvider *ui_provider);
-void ags_ui_provider_set_automation_window(AgsUiProvider *ui_provider,
-					   GtkWidget *automation_window);
-
-GtkWidget* ags_ui_provider_get_wave_window(AgsUiProvider *ui_provider);
-void ags_ui_provider_set_wave_window(AgsUiProvider *ui_provider,
-				     GtkWidget *wave_window);
-  
-GtkWidget* ags_ui_provider_get_sheet_window(AgsUiProvider *ui_provider);
-void ags_ui_provider_set_sheet_window(AgsUiProvider *ui_provider,
-				      GtkWidget *sheet_window);
   
 GtkWidget* ags_ui_provider_get_export_window(AgsUiProvider *ui_provider);
 void ags_ui_provider_set_export_window(AgsUiProvider *ui_provider,
@@ -239,8 +216,6 @@ GtkWidget* ags_ui_provider_get_composite_editor(AgsUiProvider *ui_provider);
 void ags_ui_provider_set_composite_editor(AgsUiProvider *ui_provider,
 					  GtkWidget *composite_editor);  
 
-gboolean ags_ui_provider_use_composite_editor(AgsUiProvider *ui_provider);
-
 GtkWidget* ags_ui_provider_get_navigation(AgsUiProvider *ui_provider);
 void ags_ui_provider_set_navigation(AgsUiProvider *ui_provider,
 				    GtkWidget *navigation);
