@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2021 Joël Krähemann
+ * Copyright (C) 2005-2022 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -21,8 +21,6 @@
 
 #include <ags/audio/ags_audio_signal.h>
 #include <ags/audio/ags_audio_buffer_util.h>
-#include <ags/audio/ags_generic_pitch_util.h>
-#include <ags/audio/ags_sf2_synth_util.h>
 
 #include <ags/audio/file/ags_audio_container.h>
 #include <ags/audio/file/ags_audio_container_manager.h>
@@ -275,12 +273,10 @@ ags_sf2_synth_generator_class_init(AgsSF2SynthGeneratorClass *sf2_synth_generato
    * 
    * Since: 3.9.0
    */
-  param_spec = g_param_spec_uint("pitch-type",
+  param_spec = g_param_spec_string("pitch-type",
 				 i18n_pspec("using pitch type"),
 				 i18n_pspec("The pitch type to be used"),
-				 0,
-				 G_MAXUINT32,
-				 AGS_FLUID_4TH_ORDER_INTERPOLATE,
+				 AGS_SF2_SYNTH_GENERATOR_DEFAULT_PITCH_TYPE,
 				 G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_PITCH_TYPE,
@@ -413,15 +409,15 @@ ags_sf2_synth_generator_class_init(AgsSF2SynthGeneratorClass *sf2_synth_generato
 				  param_spec);
 
   /**
-   * AgsSF2SynthGenerator:base_key:
+   * AgsSF2SynthGenerator:base-key:
    *
-   * The base_key to be used.
+   * The base key to be used.
    * 
    * Since: 3.4.0
    */
-  param_spec = g_param_spec_double("base_key",
-				   i18n_pspec("using base_key"),
-				   i18n_pspec("The base_key to be used"),
+  param_spec = g_param_spec_double("base-key",
+				   i18n_pspec("using base key"),
+				   i18n_pspec("The base key to be used"),
 				   -65535.0,
 				   65535.0,
 				   AGS_SF2_SYNTH_GENERATOR_DEFAULT_BASE_KEY,
@@ -457,7 +453,7 @@ ags_sf2_synth_generator_init(AgsSF2SynthGenerator *sf2_synth_generator)
   sf2_synth_generator->buffer_size = ags_soundcard_helper_config_get_buffer_size(config);
   sf2_synth_generator->format = ags_soundcard_helper_config_get_format(config);
 
-  sf2_synth_generator->pitch_type = AGS_FLUID_4TH_ORDER_INTERPOLATE;
+  sf2_synth_generator->pitch_type = g_strdup(AGS_SF2_SYNTH_GENERATOR_DEFAULT_PITCH_TYPE);;
 
   /* more base init */
   sf2_synth_generator->frame_count = 0;
@@ -472,11 +468,7 @@ ags_sf2_synth_generator_init(AgsSF2SynthGenerator *sf2_synth_generator)
   sf2_synth_generator->base_key = AGS_SF2_SYNTH_GENERATOR_DEFAULT_BASE_KEY;
   sf2_synth_generator->tuning = AGS_SF2_SYNTH_GENERATOR_DEFAULT_TUNING;
 
-  sf2_synth_generator->sf2 = (AgsSF2 *) g_new0(AgsSF2,
-					       1);
-
-  sf2_synth_generator->sf2->generic_pitch = (gpointer) g_new0(AgsGenericPitch,
-							      1);
+  sf2_synth_generator->sf2_synth_util = ags_sf2_synth_util_alloc();
   
   /* timestamp */
   sf2_synth_generator->timestamp = NULL;
@@ -618,9 +610,9 @@ ags_sf2_synth_generator_set_property(GObject *gobject,
   break;
   case PROP_PITCH_TYPE:
   {
-    guint pitch_type;
+    gchar *pitch_type;
       
-    pitch_type = g_value_get_uint(value);
+    pitch_type = g_value_get_string(value);
 
     ags_sf2_synth_generator_set_pitch_type(sf2_synth_generator,
 					   pitch_type);
@@ -805,7 +797,7 @@ ags_sf2_synth_generator_get_property(GObject *gobject,
   {
     g_rec_mutex_lock(sf2_synth_generator_mutex);
 
-    g_value_set_uint(value, sf2_synth_generator->pitch_type);
+    g_value_set_string(value, sf2_synth_generator->pitch_type);
 
     g_rec_mutex_unlock(sf2_synth_generator_mutex);
   }
@@ -1448,13 +1440,13 @@ ags_sf2_synth_generator_set_format(AgsSF2SynthGenerator *sf2_synth_generator, gu
  * 
  * Since: 3.9.0
  */
-guint
+gchar*
 ags_sf2_synth_generator_get_pitch_type(AgsSF2SynthGenerator *sf2_synth_generator)
 {
-  guint pitch_type;
+  gchar *pitch_type;
   
   if(!AGS_IS_SF2_SYNTH_GENERATOR(sf2_synth_generator)){
-    return(0);
+    return(NULL);
   }
 
   g_object_get(sf2_synth_generator,
@@ -1474,7 +1466,7 @@ ags_sf2_synth_generator_get_pitch_type(AgsSF2SynthGenerator *sf2_synth_generator
  * Since: 3.9.0
  */
 void
-ags_sf2_synth_generator_set_pitch_type(AgsSF2SynthGenerator *sf2_synth_generator, guint pitch_type)
+ags_sf2_synth_generator_set_pitch_type(AgsSF2SynthGenerator *sf2_synth_generator, gchar *pitch_type)
 {
   GRecMutex *sf2_synth_generator_mutex;
 
@@ -1487,7 +1479,7 @@ ags_sf2_synth_generator_set_pitch_type(AgsSF2SynthGenerator *sf2_synth_generator
 
   g_rec_mutex_lock(sf2_synth_generator_mutex);
 
-  sf2_synth_generator->pitch_type = pitch_type;
+  sf2_synth_generator->pitch_type = g_strdup(pitch_type);
 
   g_rec_mutex_unlock(sf2_synth_generator_mutex);
 }
@@ -2143,43 +2135,54 @@ ags_sf2_synth_generator_compute_instrument(AgsSF2SynthGenerator *sf2_synth_gener
   }
 #endif
 
-#if 0
-  ags_sf2_synth_util_copy(buffer,
-			  frame_count,
-			  ipatch_sample,
-			  (gdouble) note,
-			  volume,
-			  samplerate, audio_buffer_util_format,
-			  0, frame_count,
-			  AGS_SF2_SYNTH_UTIL_LOOP_NONE,
-			  0, 0);
-#else
   g_rec_mutex_lock(sf2_synth_generator_mutex);
-  
-  sf2_synth_generator->sf2->note = note;
-  
-  sf2_synth_generator->sf2->volume = volume;
 
-  sf2_synth_generator->sf2->samplerate = samplerate;
-
-  sf2_synth_generator->sf2->offset = 0;
-  sf2_synth_generator->sf2->n_frames = frame_count;
-
-  sf2_synth_generator->sf2->loop_mode = AGS_SF2_SYNTH_UTIL_LOOP_NONE;
+  sf2_synth_generator->sf2_synth_util->flags |= AGS_SF2_SYNTH_UTIL_COMPUTE_INSTRUMENT;
   
-  sf2_synth_generator->sf2->loop_start = 0;
-  sf2_synth_generator->sf2->loop_end = 0;
+  ags_sf2_synth_util_set_sf2_file(sf2_synth_generator->sf2_synth_util,
+				  audio_container);
 
-  AGS_GENERIC_PITCH(sf2_synth_generator->sf2->generic_pitch)->pitch_type = sf2_synth_generator->pitch_type;
+  ags_sf2_synth_util_set_source(sf2_synth_generator->sf2_synth_util,
+				buffer);
+  ags_sf2_synth_util_set_source_stride(sf2_synth_generator->sf2_synth_util,
+				       1);
+
+  ags_sf2_synth_util_set_buffer_length(sf2_synth_generator->sf2_synth_util,
+				       frame_count);
+
+  ags_sf2_synth_util_set_format(sf2_synth_generator->sf2_synth_util,
+				format);
+
+  ags_sf2_synth_util_set_samplerate(sf2_synth_generator->sf2_synth_util,
+				    samplerate);
   
-  ags_sf2_util_copy(sf2_synth_generator->sf2,
-		    buffer,
-		    frame_count,
-		    ipatch_sample,
-		    audio_buffer_util_format);
+  ags_sf2_synth_util_set_preset(sf2_synth_generator->sf2_synth_util,
+				preset);
+
+  ags_sf2_synth_util_set_instrument(sf2_synth_generator->sf2_synth_util,
+				    instrument);
+
+  ags_sf2_synth_util_set_note(sf2_synth_generator->sf2_synth_util,
+			      note);
+
+  ags_sf2_synth_util_set_volume(sf2_synth_generator->sf2_synth_util,
+				volume);
+
+  ags_sf2_synth_util_set_frame_count(sf2_synth_generator->sf2_synth_util,
+				     frame_count);
+
+  ags_sf2_synth_util_set_offset(sf2_synth_generator->sf2_synth_util,
+				0);
   
+  ags_sf2_synth_util_set_loop_start(sf2_synth_generator->sf2_synth_util,
+				    0);
+
+  ags_sf2_synth_util_set_loop_end(sf2_synth_generator->sf2_synth_util,
+				  0);
+
+  ags_sf2_synth_util_compute(sf2_synth_generator->sf2_synth_util);
+
   g_rec_mutex_unlock(sf2_synth_generator_mutex);    
-#endif
   
   copy_mode = ags_audio_buffer_util_get_copy_mode(audio_buffer_util_format,
 						  audio_buffer_util_format);
@@ -2402,43 +2405,54 @@ ags_sf2_synth_generator_compute_midi_locale(AgsSF2SynthGenerator *sf2_synth_gene
   buffer = ags_stream_alloc(frame_count,
 			    format);
 
-#if 0
-  ags_sf2_synth_util_copy(buffer,
-			  frame_count,
-			  ipatch_sample,
-			  (gdouble) note,
-			  volume,
-			  samplerate, audio_buffer_util_format,
-			  0, frame_count,
-			  AGS_SF2_SYNTH_UTIL_LOOP_NONE,
-			  0, 0);
-#else
   g_rec_mutex_lock(sf2_synth_generator_mutex);
   
-  sf2_synth_generator->sf2->note = note;
+  sf2_synth_generator->sf2_synth_util->flags |= AGS_SF2_SYNTH_UTIL_COMPUTE_MIDI_LOCALE;
+
+  ags_sf2_synth_util_set_sf2_file(sf2_synth_generator->sf2_synth_util,
+				  audio_container);
+
+  ags_sf2_synth_util_set_source(sf2_synth_generator->sf2_synth_util,
+				buffer);
+  ags_sf2_synth_util_set_source_stride(sf2_synth_generator->sf2_synth_util,
+				       1);
+
+  ags_sf2_synth_generator_set_bank(sf2_synth_generator->sf2_synth_util,
+				   bank);
+
+ ags_sf2_synth_generator_set_program(sf2_synth_generator->sf2_synth_util,
+				     program);
+
+  ags_sf2_synth_util_set_buffer_length(sf2_synth_generator->sf2_synth_util,
+				       frame_count);
+
+  ags_sf2_synth_util_set_format(sf2_synth_generator->sf2_synth_util,
+				format);
+
+  ags_sf2_synth_util_set_samplerate(sf2_synth_generator->sf2_synth_util,
+				    samplerate);
   
-  sf2_synth_generator->sf2->volume = volume;
+  ags_sf2_synth_util_set_note(sf2_synth_generator->sf2_synth_util,
+			      note);
 
-  sf2_synth_generator->sf2->samplerate = samplerate;
+  ags_sf2_synth_util_set_volume(sf2_synth_generator->sf2_synth_util,
+				volume);
 
-  sf2_synth_generator->sf2->offset = 0;
-  sf2_synth_generator->sf2->n_frames = frame_count;
+  ags_sf2_synth_util_set_frame_count(sf2_synth_generator->sf2_synth_util,
+				     frame_count);
 
-  sf2_synth_generator->sf2->loop_mode = AGS_SF2_SYNTH_UTIL_LOOP_NONE;
+  ags_sf2_synth_util_set_offset(sf2_synth_generator->sf2_synth_util,
+				0);
   
-  sf2_synth_generator->sf2->loop_start = 0;
-  sf2_synth_generator->sf2->loop_end = 0;
+  ags_sf2_synth_util_set_loop_start(sf2_synth_generator->sf2_synth_util,
+				    0);
 
-  AGS_GENERIC_PITCH(sf2_synth_generator->sf2->generic_pitch)->pitch_type = sf2_synth_generator->pitch_type;
-  
-  ags_sf2_util_copy(sf2_synth_generator->sf2,
-		    buffer,
-		    frame_count,
-		    ipatch_sample,
-		    audio_buffer_util_format);
+  ags_sf2_synth_util_set_loop_end(sf2_synth_generator->sf2_synth_util,
+				  0);
+
+  ags_sf2_synth_util_compute(sf2_synth_generator->sf2_synth_util);
   
   g_rec_mutex_unlock(sf2_synth_generator_mutex);    
-#endif
 
   copy_mode = ags_audio_buffer_util_get_copy_mode(audio_buffer_util_format,
 						  audio_buffer_util_format);
